<?php

/**
 * FromCrud (Class)
 *
 * Esta classe tem como objetivo linkar uma entidade a um conjunto de templates para disponibilizar um
 * formulário para operações de edição e criação de uma entidade existente ou nova.
 *
 * Localização das templates, por padrão na pasta "tpl"
 *
 * Os templates são selecionado com base no atributo "input" dos campos de uma entidade.
 *
 * Os Template selecionados devem conter a extensão "tpl" para funcionamento com a biblioteca Smarty
 *      tpl (folder default) ->
 *          input.tpl
 *
 * Templates podem recebem uma personalização "tpl_personalizado" (folder), sendo assim
 * necessário evidenciar a personalização ao instanciar a classe:
 *      $form = new Form('personalizado');
 *
 * ou então setando o design desejado logo após instanciar:
 *      $form = new Form();
 *      $form->setDesign('personalizado');
 *
 * Requer as dependências:
 * Conn-Crud -> para manipulação do Banco
 * Entity -> para controle da entidade
 * Smarty -> motor de template
 */

namespace FormCrud;

use ConnCrud\Read;
use Entity\Entity;
use EntityForm\Metadados;
use Helpers\Template;

class Form
{
    private $entity;
    private $autoSave = true;
    private $callback;
    private $fields;
    private $children;
    private $error;
    private $design = "input";

    /**
     *
     * @param string $entity
     */
    public function __construct(string $entity = null)
    {
        if ($entity)
            $this->setEntity($entity);
    }

    /**
     * @param mixed $autoSave
     */
    public function setAutoSave($autoSave = null)
    {
        $this->autoSave = $autoSave === null ? !$this->autoSave : $autoSave;
    }

    /**
     * @param mixed $callback
     */
    public function setCallback($callback)
    {
        $this->callback = $callback;
    }

    public function setChildren()
    {
        $this->children = true;
    }

    /**
     * @param mixed $fields
     */
    public function setFields($fields)
    {
        $this->fields = $fields;
    }

    /**
     * @param string $entity
     */
    public function setEntity(string $entity)
    {
        $this->entity = $entity;
    }

    /**
     * @param mixed $design
     */
    public function setDesign($design)
    {
        $this->design = $design;
    }

    /**
     * @return mixed
     */
    public function getError()
    {
        return $this->error;
    }

    public function getFormChildren($id = null, $fields = null)
    {
        $this->setChildren();
        return $this->getForm($id, $fields);
    }

    public function showFormChildren($id = null, $fields = null)
    {
        echo $this->getFormChildren($id, $fields);
    }

    public function getForm($id = null, $fields = null)
    {
        if (Entity::checkPermission($this->entity, $id)) {
            if ($id && is_array($id) && !$fields) {
                $this->setFields($id);
                $id = null;
            } elseif ($fields && is_array($fields)) {
                $this->setFields($fields);
            }

            $template = new Template("form-crud");
            $form['inputs'] = $this->prepareInputs($this->entity, "dados.", $this->readValues($this->entity, $id));
            $form['id'] = $id;
            $form['entity'] = $this->entity;
            $form['autoSave'] = $this->autoSave;
            $form['callback'] = $this->callback;
            $form['home'] = defined("HOME") ? HOME : "";
            $form['cache'] = date("YmdHi");

            return $this->scripts() . "<div class='form-control row font-large'>" . $template->getShow("form", $form) . "</div>";
        }

        return Entity::getError()[$this->entity]['id'] ?? "Permissão Negada";
    }

    public function showForm($id = null, $fields = null)
    {
        echo $this->getForm($id, $fields);
    }

    /**
     * Retorna os Scripts do form
     * @return string
     */
    private function scripts(): string
    {
        if (!$this->children)
            return "<input type='hidden' id='fields-{$this->entity}' value='" . ($this->fields ? json_encode($this->fields) : "") . "' />";

        return "";
    }

    /**
     * @param mixed $id
     * @return bool
     */
    private function notAllowForm($id): bool
    {
        if (!is_numeric($id))
            return false;

        $rules = json_decode(file_get_contents(PATH_HOME . "vendor/conn/form-crud/rules/rules.json"), true);

        if (!empty($rules)) {
            $read = new Read();
            foreach ($rules as $rule) {
                $entity = $rule[0];
                $where = "WHERE id = :id";

                if (!empty($rule[1])) {
                    if (is_array($rule[1]) && !isset($rule[2])) {
                        foreach ($rule[1] as $r) {
                            if (!empty($r[0]) && !empty($r[1]))
                                $where .= " && {$r[0]} " . (!empty($_SESSION['userlogin'][$r[1]]) ? $_SESSION['userlogin'][$r[1]] : $r[1]);
                        }
                    } elseif (!empty($rule[2])) {
                        $where .= " && {$rule[1]} " . (!empty($_SESSION['userlogin'][$rule[2]]) ? $_SESSION['userlogin'][$rule[2]] : $rule[2]);
                    }
                }

                $read->exeRead($entity, $where, "id={$id}");
                if ($read->getResult())
                    return true;
            }
        }

        return false;
    }

    /**
     * @param string $entity
     * @param int $id
     * @return mixed
     */
    private function readValues(string $entity, int $id = null)
    {
        Entity::setError(null);
        $value = Entity::read($entity, $id);
        if (Entity::getError())
            $value = Entity::read($entity);

        return Entity::getError() ? null : $this->fixValues($value);
    }

    private function fixValues($value)
    {
        foreach (Metadados::getDicionario($this->entity) as $i => $data) {
            if ($data['format'] === "datetime" && is_string($value[$data['column']]))
                $value[$data['column']] = str_replace(' ', 'T', $value[$data['column']]);
        }

        return $value;
    }

    /**
     * @param string $entity
     * @param string $ngmodel
     * @param mixed $values
     *
     * @return array
     */
    private function prepareInputs(string $entity, string $ngmodel, $values = null): array
    {
        $dados = [];
        $values = $values ?? [];

        $rel = Metadados::getRelevant($this->entity);
        $dic = Metadados::getDicionario($entity, true);

        $dados[] = "<input type='hidden' rel='title' value='{$dic[$rel]['column']}'>";

        $allowEditLoginSetor = ($entity !== "login" || (!empty($values['id']) && $_SESSION['userlogin']['id'] != $values['id']));

        foreach ($dic as $i => $data) {
            if(!empty($values['id']) && !$allowEditLoginSetor && in_array($data['column'], ["status", "setor", "nivel"]))
                continue;

            if ($data['key'] === "identifier" || (!$this->fields && $data['form']) || ($this->fields && in_array($data['column'], $this->fields))) {
                $data['path'] = PATH_HOME;
                $data['home'] = HOME;
                $data['entity'] = $entity;
                $data['value'] = $values[$data['column']] ?? null;
                $data['ngmodel'] = $ngmodel . $data['column'];
                if (!$data['form'])
                    $data['form'] = ['input' => "text", "style" => "", "class" => "", "cols" => "12", "colm" => "", "coll" => ""];

                $data = $this->checkListData($data);
                $data = $this->checkListMultData($data);
                $data = $this->checkDateValue($data);
                $data['mult'] = $this->checkExtendMultSelect($data, $values);
                $dados[] = $this->processaInput($data, $ngmodel, $values);
            }
        }

        return $dados;
    }

    private function checkDateValue($data)
    {
        if ($data['format'] === "datetime")
            $data['value'] = str_replace(' ', 'T', $data['value']);

        return $data;
    }

    private function checkListData(array $data)
    {
        if ($data['key'] === "list" || $data['key'] === "selecao") {
            $dic = Metadados::getDicionario($data['relation']);
            $rel = Metadados::getRelevant($data['relation'], true);
            $type = $rel[1];
            $rel = $rel[0];

            $data['icon'] = '<i class="material-icons padding-medium">' . $this->getIcons($type) . '</i>';
            $data['title'] = !empty($rel) && $data['value'] ? $data['value'][$dic[$rel]['column']] : "";
            $data['id'] = isset($data['value']['id']) ? $data['value']['id'] : "";
        }

        return $data;
    }

    private function checkListMultData(array $data)
    {
        if ($data['key'] === "list_mult" || $data['key'] === "extend_mult" || $data['key'] === "selecao_mult") {
            $dic = Metadados::getDicionario($data['relation']);
            $rel = Metadados::getRelevant($data['relation'], true);
            $type = $rel[1];
            $rel = $rel[0];

            $data['icon'] = '<i class="material-icons padding-medium">' . $this->getIcons($type) . '</i>';
            $data['info']['title'] = (!empty($rel) ? $dic[$rel]['column'] : "");
        }

        return $data;
    }

    /**
     * Busca por campos mult relacionais que precisam ser selecionados
     *
     * @param array $data
     * @param array $values
     * @return string
     */
    private function checkExtendMultSelect(array $data, array $values): string
    {
        $mult = "";
        if (in_array($data['key'], ["list_mult", "extend_mult", "selecao_mult", "list", "extend", "selecao"]) && !empty($data['select'])) {

            $dicionario = \EntityForm\Metadados::getDicionario($data['relation']);
            foreach ($data['select'] as $select) {
                foreach ($dicionario as $item) {
                    if ($item['column'] === $select) {
                        if (!empty($values[$select . "__" . $data['column']])) {
                            $value_select = $this->readValues($item['relation'], $values[$select . "__" . $data['column']]);
                            $dicSelecao = Metadados::getDicionario($item['relation']);
                            $relevant = Metadados::getRelevant($item['relation']);
                            $item['title'] = $value_select[$dicSelecao[$relevant]['column']];
                            $item['id'] = $values[$select . "__" . $data['column']];
                        } else {
                            $item['title'] = "";
                            $item['id'] = "";
                        }

                        $item['nome'] = preg_match('/s$/i', $item['nome']) ? substr($item['nome'], 0, strlen($item['nome']) - 1) : $item['nome'];
                        $item['genero'] = preg_match('/a$/i', $item['nome']) ? "a" : "o";
                        $item['parentColumn'] = $data['column'];
                        $item['parentValue'] = $values[$data['column']] ? 1 : "";
                        $item['column'] = $item['column'] . "__" . $data['column'];
                        $item['ngmodel'] = "dados." . $item['column'];
                        $item['entity'] = $data['relation'];
                        $tpl = new \Helpers\Template("form-crud");

                        $mult .= $tpl->getShow("selecaoUnique", $item);

                        break;
                    }
                }
            }
        }

        return $mult;
    }

    private function getIcons($type)
    {
        switch ($type) {
            case 'email':
                return "email";
                break;
            case 'tel':
                return "settings_phone";
                break;
            case 'cep':
                return "location_on";
                break;
            default:
                return "folder";
        }
    }

    /**
     * @param array $data
     * @param string $ngmodel
     * @param mixed $value
     * @return mixed
     */
    private function processaInput(array $data, string $ngmodel, $value)
    {
        $template = new Template("form-crud");

        if ($data['key'] === "extend") {
            return $this->getExtended($data['column'], $data['relation'], $ngmodel, $value, $data);

        } else {

            return '<div class="col '
                . (!empty($data['form']['cols']) ? 's' . $data['form']['cols'] : "") . ' '
                . (!empty($data['form']['colm']) ? 'm' . $data['form']['colm'] : "") . ' '
                . (!empty($data['form']['coll']) ? 'l' . $data['form']['coll'] : "") . ' '
                . 'margin-bottom">'
                . $template->getShow($data['form']['input'], $data)
                . '</div>';
        }
    }

    private function getExtended($column, $table, $ngmodel, $value, $form)
    {
        $templateExt = new Template("form-crud");
        $form['inputs'] = $this->prepareInputs($table, $ngmodel . $column . ".", $value[$column] ?? []);

        return $templateExt->getShow("extend", $form);
    }
}